      program evol
      implicit none
!
!     compute the analytic and some numeric solutions of eq. dT/dt=F-aT
!     for a number (nt) of time steps with increment dt
!     starting with T=T0 for t=0.
!     in addition, the difference to the analytic solution, 
!     the RMS error and the maximum error are computed
!
      integer,parameter :: nt = 400   ! no timesteps to be computed
      integer,parameter :: ns = 6     ! no solutions to be computed
!     
!     1: analytic solution
!     2: Euler explicit
!     3: implicit
!     4: Crank Nicolson
!     5: Runge Kutta
!     6: Adams Bashforth
!
      real,parameter ::    dt = 0.5   ! time increment
      real,parameter ::    t0 = 0.0   ! initial value of t
      real,parameter ::     f = 1.0   ! forcing
      real,parameter ::     a = 0.05  ! damping
      real,parameter ::     g = 0.5   ! Crank Nicolson weight
!
      integer :: jt,js                ! loop increments
      real :: t(ns) = 0.              ! t
      real :: time                    ! time
      real :: zk1,zk2,zk3,zk4         ! needed for Runge Kutta
      real :: zm1,tm1                 ! t(time-1) needed for Adams Bashford
      real :: rms(ns) = 0.            ! RMS error
      real :: rma(ns) = 0.            ! maximum error
!
!     open output file
!
      open(10,file='T.txt',form='formatted')    ! all solutions
      open(11,file='TA.txt',form='formatted')   ! difference to analytic
!
!     set initial values
!
      time=0
      t(:)=t0
      rms(:)=0.
      rma(:)=0.
      tm1=t0
!
!     print out
!
      write(10,*) time,t(:)
      write(11,*) time,t(2:ns)-t(1)
!
!     loop
!
      do jt=1,nt
       time=time+dt                                  ! increment time
!
!      solutions
!
       t(1)=f/a+(t0-f/a)*exp(-a*time)                ! analytic t(time)
       t(2)=t(2)+dt*(f-a*t(2))                       ! Euler
       t(3)=(t(3)+dt*f)/(1.+dt*a)                    ! Implicit 
       t(4)=(t(4)+dt*(f-g*a*t(4)))/(1.+(1.-g)*a*dt)  ! Crank Nicolson
       zk1=dt*(f-a*t(5))                        
       zk2=dt*(f-a*(t(5)+zk1*0.5))
       zk3=dt*(f-a*(t(5)+zk2*0.5))
       zk4=dt*(f-a*(t(5)+zk3))
       t(5)=t(5)+(zk1+2.*zk2+2.*zk3+zk4)/6.          ! Runge Kutta 
       if(jt > 1) then
        zm1=t(6)
        t(6)=t(6)+dt*(3.*(f-a*t(6))-(f-a*tm1))*0.5   ! Adams Bashford
        tm1=zm1
       else
        t(6)=t(1)                                    ! set initial t(1) for A B
       endif                                           
!   
       rms(:)=rms(:)+(t(:)-t(1))**2                  ! rms error
       rma(:)=max(rma(:),abs(t(:)-t(1)))             ! max error
!
       write(10,*) time,t(:)                         ! write results
       write(11,*) time,t(2:ns)-t(1)                 ! write errors
      enddo
!
!     close output
!
      close(10)
      close(11)
!
!     print rms and max. error
!
      rms(:)=SQRT(rms(:)/real(nt))
      do js=1,ns 
       print*,'solution: ',js,' rms= ',rms(js),' max. err.= ',rma(js)
      enddo
!
      stop
      end

